﻿/*
 * Created by SharpDevelop.
 * User: misthema
 * Date: 31.3.2017
 * Time: 17:04
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TradeServer.ScriptingDriver.DataObjects;
using TradeServer.ScriptingDriver.Interfaces;
using TicTacTec.TA.Library;

namespace PshaisIndicators
{
	/// <summary>
	/// Description of ChaikinADLine.
	/// </summary>
	public class ChaikinADLine : IIndicatorScript
    {
		int _maLen = 10;
		Core.MAType _maType = Core.MAType.Sma;
		double _chaikin, _ma, _lastMa;
		
        public string Name
        {
            get { return "pshai's Chaikin A/D Line"; }
        }
        
        public int TicksBack
        {
        	get { return Core.AdLookback() + Core.MovingAverageLookback(_maLen, _maType) + 3; }
        }

        public List<ScriptParameter> GetParameters()
        {
            List<ScriptParameter> inputField = new List<ScriptParameter>
            {
                new ScriptParameter("MA Type", ScriptParameterType.String, _maType.ToString()),
                new ScriptParameter("MA Length", ScriptParameterType.Integer, _maLen.ToString())
            };

            return inputField;
        }

        public void SetParameters(Dictionary<string, object> parameters)
        {
            foreach (var item in parameters) // lets just loop them all
            {
            	switch(item.Key)
            	{
            		case "MA Type":
            			_maType = ParseMAType( Convert.ToString(item.Value) );
            			break;
            			
            		case "MA Length":
            			_maLen = Convert.ToInt32(item.Value);
            			break;
            	}
            }
        }

        // Chart lines
        public List<ChartRecord> GetChartLines()
        {
            List<ChartRecord> chartLines = new List<ChartRecord>()
            {
            	new ChartRecord() { Name = "A/D", ChartIndex = 1, LineType = 1, HexLineColor = "#AAAAAA" },
            	new ChartRecord() { Name = _maType.ToString(), ChartIndex = 1, LineType = 0, HexLineColor = "#FF6666" }
            };

            return chartLines;
        }
        public List<decimal> GetChartData()
        {
            List<decimal> chartData = new List<decimal>()
            {
            	Convert.ToDecimal(_chaikin),
            	Convert.ToDecimal(_ma)
            };
            

            return chartData;
        }

        public void Init()
        {
            
        }

        public IndicatorResult GetResult(IndicatorContext context)
        {
            try
            {
                IPriceInstrument instrument = context.PriceInstrument;
                int position = instrument.Close.Count() - (1 + context.Offset);
                
                var high = instrument.High;
                var low = instrument.Low;
                var close = instrument.Close;
                var volume = instrument.Volume;
                
                // Chaikin A/D Line
                int adBeginIdx, adOutLen;
                var adOutReal = new double[position];
                var adRet = Core.Ad(0, position-1, high, low, close, volume, out adBeginIdx, out adOutLen, adOutReal);
                
                if( adRet != Core.RetCode.Success )
                	throw new Exception(string.Format("Chaikin A/D failed! RetCode: {0}", adRet));
                
                /*
                context.Logger.LogToFile("AD:");
                context.Logger.LogToFile(string.Format("{0}", position));
                context.Logger.LogToFile(string.Format("{0}", adBeginIdx));
                context.Logger.LogToFile(string.Format("{0}", adOutLen));
                context.Logger.LogToFile(string.Format("{0}", adOutReal.Length));
                */
                
                // Moving Average
                int maBeginIdx, maOutLen;
                var maOutReal = new double[position];
                var maRet = Core.MovingAverage(0, position-1, adOutReal, _maLen, _maType, out maBeginIdx, out maOutLen, maOutReal);
                
                if( maRet != Core.RetCode.Success )
                	throw new Exception(string.Format("Moving Average failed! RetCode: {0}", maRet));
                
                _chaikin = adOutReal[adOutLen -1];
                _ma = maOutReal[maOutLen -1];
                _lastMa = maOutReal[maOutLen -2];
                
                if(_lastMa < _ma) // going up
                	return IndicatorResult.BuyLong;
                
                if(_lastMa > _ma) // going down
                	return IndicatorResult.SellShort;

            }
            catch (Exception ex)
            {
                context.Logger.LogToFile(string.Format("Exception detected: {0}", ex.Message));
            }

            return IndicatorResult.StayNoPosition;
        }
        
        
        // Annoyingly enough, this is needed where ever it's used - doesn't work from another class-file! :-(
        public static Core.MAType ParseMAType(string value)
		{
			Core.MAType val;
			switch( value.ToLower() )
			{
				case "sma":
					val = Core.MAType.Sma;
					break;
               	case "ema":
					val = Core.MAType.Ema;
					break;
               	case "dema":
					val = Core.MAType.Dema;
					break;
               	case "tema":
					val = Core.MAType.Tema;
					break;
               	case "trima":
					val = Core.MAType.Trima;
					break;
               	case "t3":
					val = Core.MAType.T3;
					break;
               	case "mama":
					val = Core.MAType.Mama;
					break;
               	case "kama":
					val = Core.MAType.Kama;
					break;
				case "wma":
					val = Core.MAType.Wma;
					break;
				default:
					val = Core.MAType.Sma;
					break;
			}
			
			return val;
		}
	}
}
